@@ -2032,6 +2032,9 @@ bool pkgDPkgPM::Go(APT::Progress::PackageManager *progress)
 	    && dynamic_cast<debSystem*>(_system)->IsLocked() == true) {
 	    setenv("DPKG_FRONTEND_LOCKED", "true", 1);
 	 }
+	 if (_config->Find("DPkg::Path", "").empty() == false)
+	    setenv("PATH", _config->Find("DPkg::Path", "").c_str(), 1);
+
 	 execvp(Args[0], (char**) &Args[0]);
 	 cerr << "Could not exec dpkg!" << endl;
 	 _exit(100);
