@@ -1,6 +1,5 @@
 // -*- mode: cpp; mode: fold -*-
 // Description								/*{{{*/
-// $Id: http.cc,v 1.59 2004/05/08 19:42:35 mdz Exp $
 /* ######################################################################
 
    HTTP Acquire Method - This is the HTTP acquire method for APT.
@@ -26,6 +25,7 @@
 #include <apt-pkg/proxy.h>
 #include <apt-pkg/strutl.h>
 
+#include <chrono>
 #include <cstring>
 #include <iostream>
 #include <sstream>
@@ -50,7 +50,7 @@ using namespace std;
 
 unsigned long long CircleBuf::BwReadLimit=0;
 unsigned long long CircleBuf::BwTickReadData=0;
-struct timeval CircleBuf::BwReadTick={0,0};
+std::chrono::steady_clock::duration CircleBuf::BwReadTick{0};
 const unsigned int CircleBuf::BW_HZ=10;
 
 // CircleBuf::CircleBuf - Circular input buffer				/*{{{*/
@@ -99,18 +99,17 @@ bool CircleBuf::Read(std::unique_ptr<MethodFd> const &Fd)
       unsigned long long const BwReadMax = CircleBuf::BwReadLimit/BW_HZ;
 
       if(CircleBuf::BwReadLimit) {
-	 struct timeval now;
-	 gettimeofday(&now,0);
+	 auto const now = std::chrono::steady_clock::now().time_since_epoch();
+	 auto const d = now - CircleBuf::BwReadTick;
 
-	 unsigned long long d = (now.tv_sec-CircleBuf::BwReadTick.tv_sec)*1000000 +
-	    now.tv_usec-CircleBuf::BwReadTick.tv_usec;
-	 if(d > 1000000/BW_HZ) {
+	 auto const tickLen = std::chrono::microseconds(std::chrono::seconds(1)) / BW_HZ;
+	 if(d > tickLen) {
 	    CircleBuf::BwReadTick = now;
 	    CircleBuf::BwTickReadData = 0;
-	 } 
-	 
+	 }
+
 	 if(CircleBuf::BwTickReadData >= BwReadMax) {
-	    usleep(1000000/BW_HZ);
+	    usleep(tickLen.count());
 	    return true;
 	 }
       }
@@ -135,8 +134,6 @@ bool CircleBuf::Read(std::unique_ptr<MethodFd> const &Fd)
 	 return false;
       }
 
-      if (InP == 0)
-	 gettimeofday(&Start,0);
       InP += Res;
    }
 }
@@ -268,21 +265,6 @@ bool CircleBuf::Write(string &Data)
    return true;
 }
 									/*}}}*/
-// CircleBuf::Stats - Print out stats information			/*{{{*/
-// ---------------------------------------------------------------------
-/* */
-void CircleBuf::Stats()
-{
-   if (InP == 0)
-      return;
-   
-   struct timeval Stop;
-   gettimeofday(&Stop,0);
-/*   float Diff = Stop.tv_sec - Start.tv_sec + 
-             (float)(Stop.tv_usec - Start.tv_usec)/1000000;
-   clog << "Got " << InP << " in " << Diff << " at " << InP/Diff << endl;*/
-}
-									/*}}}*/
 CircleBuf::~CircleBuf()							/*{{{*/
 {
    delete [] Buf;
@@ -338,14 +320,14 @@ static ResultState UnwrapHTTPConnect(std::string Host, int Port, URI Proxy, std:
    std::string ProperHost;
 
    if (Host.find(':') != std::string::npos)
-      ProperHost = '[' + Host + ']';
+      ProperHost = '[' + Proxy.Host + ']';
    else
-      ProperHost = Host;
+      ProperHost = Proxy.Host;
 
    // Build the connect
    Req << "CONNECT " << Host << ":" << std::to_string(Port) << " HTTP/1.1\r\n";
    if (Proxy.Port != 0)
-      Req << "Host: " << ProperHost << ":" << std::to_string(Port) << "\r\n";
+      Req << "Host: " << ProperHost << ":" << std::to_string(Proxy.Port) << "\r\n";
    else
       Req << "Host: " << ProperHost << "\r\n";
 
@@ -992,6 +974,10 @@ void HttpMethod::SendReq(FetchItem *Itm)
    Req << "User-Agent: " << ConfigFind("User-Agent",
 		"Debian APT-HTTP/1.3 (" PACKAGE_VERSION ")") << "\r\n";
 
+   auto const referer = ConfigFind("Referer", "");
+   if (referer.empty() == false)
+      Req << "Referer: " << referer << "\r\n";
+
    Req << "\r\n";
 
    if (Debug == true)
